@@ -25,12 +25,8 @@
 extern char *program_name;
 
 void
-usage (int status, const char *reason)
+usage (int status)
 {
-  if (reason != NULL)
-    fprintf (status == 0 ? stdout : stderr, "%s: %s\n",
-	     program_name, reason);
-
   if (status != 0)
     fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
@@ -51,16 +47,18 @@ Copy SOURCE to DEST, or multiple SOURCE(s) to DIRECTORY.\n\
   -i, --interactive            prompt before overwrite\n\
   -l, --link                   link files instead of copying\n\
   -p, --preserve               preserve file attributes if possible\n\
+  -P, --parents                append source path to DIRECTORY\n\
   -r                           copy recursively, non-directories as files\n\
       --sparse=WHEN            control creation of sparse files\n\
-  -s, --symbolic-link          make symbolic links instead of copying\n\
-  -u, --update                 copy only older or brand new files\n\
-  -v, --verbose                explain what is being done\n\
-  -x, --one-file-system        stay on this file system\n\
-  -P, --parents                append source path to DIRECTORY\n\
   -R, --recursive              copy directories recursively\n\
+  -s, --symbolic-link          make symbolic links instead of copying\n\
   -S, --suffix=SUFFIX          override the usual backup suffix\n\
+  -u, --update                 copy only when the SOURCE file is newer\n\
+                                 than the destination file or when the\n\
+                                 destination file is missing\n\
+  -v, --verbose                explain what is being done\n\
   -V, --version-control=WORD   override the usual version control\n\
+  -x, --one-file-system        stay on this file system\n\
       --help                   display this help and exit\n\
       --version                output version information and exit\n\
 \n\
@@ -83,7 +81,9 @@ version control may be set with VERSION_CONTROL, values are:\n\
 \n\
 As a special case, cp makes a backup of SOURCE when the force and backup\n\
 options are given and SOURCE and DEST are the same name for an existing,\n\
-regular file.\n"));
+regular file.\n\
+"));
+      puts (_("\nReport bugs to fileutils-bugs@gnu.ai.mit.edu"));
     }
   exit (status);
 }
