@@ -36,12 +36,6 @@ int link ();			/* Some systems don't declare this anywhere. */
 int symlink ();
 #endif
 
-#ifdef S_ISLNK
-# define LINK_TYPE symbolic_link ? _("symbolic link") : _("hard link")
-#else
-# define LINK_TYPE ""
-#endif
-
 /* Construct a string NEW_DEST by concatenating DEST, a slash, and
    basename(SOURCE) in alloca'd memory.  Don't modify DEST or SOURCE.  */
 
@@ -80,6 +74,10 @@ static int (*linkfunc) ();
 /* If nonzero, make symbolic links; otherwise, make hard links.  */
 static int symbolic_link;
 
+/* A string describing type of link to make.  For use in verbose
+   diagnostics and in error messages.  */
+static const char *link_type_string;
+
 /* If nonzero, ask the user before removing existing files.  */
 static int interactive;
 
@@ -289,15 +287,14 @@ do_link (const char *source, const char *dest)
     }
 
   if (verbose)
-    printf (_("create %s %s to %s\n"), LINK_TYPE,
-	    dest, source);
+    printf (_("create %s %s to %s\n"), link_type_string, dest, source);
 
   if ((*linkfunc) (source, dest) == 0)
     {
       return 0;
     }
 
-  error (0, errno, _("cannot create %s `%s' to `%s'"), LINK_TYPE,
+  error (0, errno, _("cannot create %s `%s' to `%s'"), link_type_string,
 	 dest, source);
 
   if (dest_backup)
@@ -332,8 +329,8 @@ Makes hard links by default, symbolic links with -s.\n\
                                  directory as if it were a normal file\n\
   -i, --interactive            prompt whether to remove destinations\n\
   -s, --symbolic               make symbolic links instead of hard links\n\
-  -v, --verbose                print name of each file before linking\n\
   -S, --suffix=SUFFIX          override the usual backup suffix\n\
+  -v, --verbose                print name of each file before linking\n\
   -V, --version-control=WORD   override the usual version control\n\
       --help                   display this help and exit\n\
       --version                output version information and exit\n\
@@ -345,7 +342,9 @@ version control may be set with VERSION_CONTROL, values are:\n\
 \n\
   t, numbered     make numbered backups\n\
   nil, existing   numbered if numbered backups exist, simple otherwise\n\
-  never, simple   always make simple backups\n"));
+  never, simple   always make simple backups\n\
+"));
+      puts (_("\nReport bugs to fileutils-bugs@gnu.ai.mit.edu"));
     }
   exit (status);
 }
@@ -368,7 +367,6 @@ main (int argc, char **argv)
     simple_backup_suffix = version;
   version = getenv ("VERSION_CONTROL");
 
-  linkfunc = link;
   symbolic_link = remove_existing_files = interactive = verbose
     = hard_dir_link = 0;
   errors = 0;
@@ -423,7 +421,7 @@ main (int argc, char **argv)
 
   if (show_version)
     {
-      printf ("ln - %s\n", PACKAGE_VERSION);
+      printf ("ln (%s) %s\n", GNU_PACKAGE, VERSION);
       exit (0);
     }
 
@@ -441,7 +439,17 @@ main (int argc, char **argv)
 
 #ifdef S_ISLNK
   if (symbolic_link)
-    linkfunc = symlink;
+    {
+      linkfunc = symlink;
+      link_type_string = _("symbolic link");
+    }
+  else
+    {
+      linkfunc = link;
+      link_type_string = _("hard link");
+    }
+#else
+  link_type_string = _("link");
 #endif
 
   if (optind == argc - 1)
